/**
 *
 */


// System Import Headers.
#include <cstdio>
#include <cstdlib>
#include <cctype>
#include <filesystem>
#include <fstream>
#include <iostream>
#include <iterator>
#include <string>
#include <vector>

// User Import Headers.
#ifndef user_headers
    #define user_headers
    #include "image.h"
    #include "utility.h"
#endif

#ifndef user_headers_logging
    #define user_headers_logging
    #include "logging.h"
#endif


// Method Declaration.
void process_dir(std::string path_str);
void process_file(std::string path_str);
// void validate_path(char* path_str);
// void compute(int a, int b);
// __global__ void cuda_add(int a, int b, int* c);


// Global Variables.
#ifndef LOGGING_CLASS_INIT
    #define LOGGING_CLASS_INIT
    Logging logger;
#endif


/*
 * Program's main. Initializes and runs program.
 */
int main(int argc, char* argv[]) {
    printf("Starting program.\n");
    printf("\n");
    printf("\n");

    // Initialize logging and clear any existing log contents for new program run.
    logger.clear_log_files();

    // Check provided number of args.
    if (argc < 2) {
        // No args provided.
        printf("Provided too few program args. Expected one arg.");
    } else if (argc > 2) {
        // More than one arg provided.
        printf("Provided too many program args. Expected one arg.");
    } else {
        // Correct number of args provided. Pass to image class for processing.

        // Check data type.
        std::string path_str = argv[1];
        path_validator_struct* return_struct = validate_path(path_str);
        int path_type = return_struct->file_type;
        if (return_struct->err_code) {
            std::cerr << "Error verifying file type: " << return_struct->err_code.message() << std::endl;
        }
        free_path_validator_struct(return_struct);

        // Handle based on provided type.
        if (path_type == TYPE_DIR) {
            // Process dir.
            process_dir(path_str);
        } else if (path_type == TYPE_FILE) {
            // Process file.
            process_file(path_str);
        }
    }

    printf("\n");
    printf("\n");
    printf("Terminating program.\n");
    exit(0);
}


/**
 * Finds and processes all files in directory.
 * Assumes directory is full of image files.
 */
void process_dir(std::string path_str) {
    logger.debug("process_dir():");
    logger.debug("    path_str: " + path_str);
    logger.debug("");

    logger.info("Handling dir.");

    // Process all files in directory.
    for (const auto & entry : std::filesystem::directory_iterator(path_str)) {
        // Get as string.
        std::string file_path_str{entry.path().u8string()};
        // std::cout << file_path_str << std::endl;

        // Check if new path value is directory or file.
        path_validator_struct* return_struct = validate_path(file_path_str);
        int path_type = return_struct->file_type;
        if (return_struct->err_code) {
            std::cerr << "Error verifying file type: " << return_struct->err_code.message() << std::endl;
        }
        free_path_validator_struct(return_struct);

        // Handle based on provided type.
        if (path_type == TYPE_DIR) {
            // Process dir.
            process_dir(file_path_str);
        } else if (path_type == TYPE_FILE) {
            // Process file.
            process_file(file_path_str);
        }

        // Send individual file path for processing.
        process_file(entry.path());
    }
}


/**
 * Processes a single image file.
 */
void process_file(std::string path_str) {
    logger.debug("process_file():");
    logger.debug("    path_str: " + path_str);
    logger.debug("");

    logger.info("Handling file.");

    Image image(path_str);
    image.display_properties();
    image.save();
}



// /*
//  * Program's main. Initializes and runs program.
//  */
// int main(int argc, char* argv[]) {
//     printf("Starting program.\n");
//     printf("\n");

//     // Check provided number of args.
//     if (argc < 3) {
//         // Less than two args provided.
//         printf("Provided too few program args. Expected two integers to add.");
//     } else if (argc > 3) {
//         // More than two args provided.
//         printf("Provided too many program args. Expected two integers to add.");
//     } else {
//         // Two args provided. Compute with GPU.
//         int a = atoi(argv[1]);
//         int b = atoi(argv[2]);
//         printf("Adding %i and %i\n", a, b);
//         compute(a, b);
//     }

//     printf("\n");
//     printf("Terminating program.\n");
//     exit(0);
// }


// /*
//  * Function to have CPU and GPU collaborate for basic addition computation.
//  */
// void compute(int a, int b) {
//     int sum;
//     int* dev_sum;

//     // Allocate memory on the GPU.
//     cudaMalloc((int**) &dev_sum, sizeof(int));

//     // Call CUDA function.
//     cuda_add<<<1,1>>>(a, b, dev_sum);

//     // Copy GPU result back to CPU.
//     cudaMemcpy(&sum, dev_sum, sizeof(int), cudaMemcpyDeviceToHost);

//     // Display result.
//     printf("Sum is %i\n", sum);

//     cudaFree(dev_sum);
// }


// /**
//  * Entrypoint to GPU kernel execution.
//  */
// __global__ void cuda_add(int a, int b, int* c) {
//     *c = a + b;
// }
